searchData={"items":[{"type":"module","title":"MSSMT","doc":"Implementation of a Merkle-Sum Sparse Merkle Tree (MS-SMT).\n\nA MS-SMT is a data structure that combines the features of a Merkle tree\nand a sum tree, allowing for efficient proofs of inclusion and accumulation\nof values.","ref":"MSSMT.html"},{"type":"function","title":"MSSMT.delete/2","doc":"Deletes a key-value pair from the MS-SMT.","ref":"MSSMT.html#delete/2"},{"type":"function","title":"Examples - MSSMT.delete/2","doc":"iex> tree = MSSMT.new()\n    iex> tree = MSSMT.insert(tree, \"key1\", 100)\n    iex> tree = MSSMT.delete(tree, \"key1\")\n    iex> MSSMT.get(tree, \"key1\")\n    nil","ref":"MSSMT.html#delete/2-examples"},{"type":"function","title":"MSSMT.generate_proof/2","doc":"Generates a proof of inclusion for a given key.\n\nThe proof consists of the necessary sibling nodes to reconstruct the root hash.","ref":"MSSMT.html#generate_proof/2"},{"type":"function","title":"Examples - MSSMT.generate_proof/2","doc":"iex> tree = MSSMT.new()\n    iex> tree = MSSMT.insert(tree, \"key1\", 100)\n    iex> proof = MSSMT.generate_proof(tree, \"key1\")\n    iex> is_list(proof)\n    true","ref":"MSSMT.html#generate_proof/2-examples"},{"type":"function","title":"MSSMT.get/2","doc":"Retrieves the value associated with a key from the MS-SMT.\n\nReturns `nil` if the key does not exist.","ref":"MSSMT.html#get/2"},{"type":"function","title":"Examples - MSSMT.get/2","doc":"iex> tree = MSSMT.new()\n    iex> MSSMT.get(tree, \"nonexistent_key\")\n    nil","ref":"MSSMT.html#get/2-examples"},{"type":"function","title":"MSSMT.insert/3","doc":"Inserts a key-value pair into the MS-SMT.","ref":"MSSMT.html#insert/3"},{"type":"function","title":"Examples - MSSMT.insert/3","doc":"iex> tree = MSSMT.new()\n    iex> tree = MSSMT.insert(tree, \"key1\", 100)\n    iex> MSSMT.get(tree, \"key1\")\n    100","ref":"MSSMT.html#insert/3-examples"},{"type":"function","title":"MSSMT.new/0","doc":"Creates a new empty MS-SMT.","ref":"MSSMT.html#new/0"},{"type":"function","title":"MSSMT.root_hash/1","doc":"Calculates the root hash of the MS-SMT.\n\nThe root hash is a cryptographic representation of the entire tree state.","ref":"MSSMT.html#root_hash/1"},{"type":"function","title":"Examples - MSSMT.root_hash/1","doc":"iex> tree = MSSMT.new()\n    iex> tree = MSSMT.insert(tree, \"key1\", 100)\n    iex> root_hash = MSSMT.root_hash(tree)\n    iex> byte_size(root_hash)\n    32","ref":"MSSMT.html#root_hash/1-examples"},{"type":"function","title":"MSSMT.total_sum/1","doc":"Calculates the total sum of all values in the MS-SMT.","ref":"MSSMT.html#total_sum/1"},{"type":"function","title":"Examples - MSSMT.total_sum/1","doc":"iex> tree = MSSMT.new()\n    iex> tree = MSSMT.insert(tree, \"key1\", 100)\n    iex> tree = MSSMT.insert(tree, \"key2\", 200)\n    iex> MSSMT.total_sum(tree)\n    300","ref":"MSSMT.html#total_sum/1-examples"},{"type":"function","title":"MSSMT.update/3","doc":"Updates the value associated with a key in the MS-SMT.\n\nIf the key does not exist, the tree remains unchanged.","ref":"MSSMT.html#update/3"},{"type":"function","title":"Examples - MSSMT.update/3","doc":"iex> tree = MSSMT.new()\n    iex> tree = MSSMT.insert(tree, \"key1\", 100)\n    iex> tree = MSSMT.update(tree, \"key1\", 200)\n    iex> MSSMT.get(tree, \"key1\")\n    200","ref":"MSSMT.html#update/3-examples"},{"type":"function","title":"MSSMT.verify_proof/4","doc":"Verifies a proof of inclusion for a given key and value.\n\nReturns `true` if the proof is valid, `false` otherwise.","ref":"MSSMT.html#verify_proof/4"},{"type":"function","title":"Examples - MSSMT.verify_proof/4","doc":"iex> tree = MSSMT.new()\n    iex> tree = MSSMT.insert(tree, \"key1\", 100)\n    iex> root_hash = MSSMT.root_hash(tree)\n    iex> proof = MSSMT.generate_proof(tree, \"key1\")\n    iex> MSSMT.verify_proof(root_hash, \"key1\", 100, proof)\n    true","ref":"MSSMT.html#verify_proof/4-examples"},{"type":"module","title":"MSSMT.Node","doc":"Represents a node in the MS-SMT.\n\nEach node contains:\n- `key`: The unique identifier for the leaf node.\n- `value`: The value associated with the key.\n- `hash`: The cryptographic hash of the node.\n- `sum`: The cumulative sum of values from the leaves up to this node.","ref":"MSSMT.Node.html"},{"type":"type","title":"MSSMT.Node.t/0","doc":"The `MSSMT.Node` struct represents a node in the Merkle-Sum Sparse Merkle Tree.","ref":"MSSMT.Node.html#t:t/0"},{"type":"extras","title":"Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"# Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir\n\nA **Merkle-Sum Sparse Merkle Tree (MS-SMT)** is a data structure that combines the features of a Merkle tree and a sum tree, allowing for efficient proofs of inclusion and accumulation of values. It's particularly useful for securely storing large amounts of data with the ability to verify the integrity and sum of the data efficiently.\n\nThis Elixir implementation provides a simple and efficient MS-SMT library for use in your projects.","ref":"readme.html"},{"type":"extras","title":"Table of Contents - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"- [Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir](#merkle-sum-sparse-merkle-tree-ms-smt-in-elixir)\n  - [Table of Contents](#table-of-contents)\n  - [Installation](#installation)\n  - [Usage](#usage)\n    - [Basic Operations](#basic-operations)\n    - [Handling Binary Keys of Different Lengths](#handling-binary-keys-of-different-lengths)\n    - [Generating and Verifying Proofs](#generating-and-verifying-proofs)\n  - [API Reference](#api-reference)\n    - [MSSMT.new/0](#mssmtnew0)\n    - [MSSMT.insert/3](#mssmtinsert3)\n    - [MSSMT.get/2](#mssmtget2)\n    - [MSSMT.update/3](#mssmtupdate3)\n    - [MSSMT.delete/2](#mssmtdelete2)\n    - [MSSMT.root\\_hash/1](#mssmtroot_hash1)\n    - [MSSMT.total\\_sum/1](#mssmttotal_sum1)\n    - [MSSMT.generate\\_proof/2](#mssmtgenerate_proof2)\n    - [MSSMT.verify\\_proof/4](#mssmtverify_proof4)\n  - [Development](#development)\n  - [Contributing](#contributing)\n  - [License](#license)","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Installation - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"To use `mssmt` in your Elixir project, add it to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mssmt, git: \"https://github.com/AbdelStark/mssmt.git\", tag: \"v0.1.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Operations - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"```elixir\n# Import the MSSMT module\nimport MSSMT\n\n# Create a new empty MS-SMT\ntree = MSSMT.new()\n\n# Insert key-value pairs\ntree = MSSMT.insert(tree, <<1, 2, 3>>, 100)\ntree = MSSMT.insert(tree, <<4, 5, 6>>, 200)\n\n# Retrieve a value\nvalue = MSSMT.get(tree, <<1, 2, 3>>)  # Returns 100\n\n# Update a value\ntree = MSSMT.update(tree, <<1, 2, 3>>, 150)\n\n# Delete a key-value pair\ntree = MSSMT.delete(tree, <<4, 5, 6>>)\n\n# Calculate root hash\nroot_hash = MSSMT.root_hash(tree)  # Returns a 32-byte binary hash\n\n# Calculate total sum\ntotal_sum = MSSMT.total_sum(tree)  # Returns 150\n```","ref":"readme.html#basic-operations"},{"type":"extras","title":"Handling Binary Keys of Different Lengths - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"```elixir\ntree = MSSMT.new()\ntree = MSSMT.insert(tree, <<1>>, 100)\ntree = MSSMT.insert(tree, <<1, 2>>, 200)\ntree = MSSMT.insert(tree, <<1, 2, 3>>, 300)\n\nMSSMT.get(tree, <<1>>)          # Returns 100\nMSSMT.get(tree, <<1, 2>>)       # Returns 200\nMSSMT.get(tree, <<1, 2, 3>>)    # Returns 300\nMSSMT.total_sum(tree)           # Returns 600\n```","ref":"readme.html#handling-binary-keys-of-different-lengths"},{"type":"extras","title":"Generating and Verifying Proofs - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"```elixir\ntree = MSSMT.new()\ntree = MSSMT.insert(tree, <<1>>, 100)\ntree = MSSMT.insert(tree, <<2>>, 200)\ntree = MSSMT.insert(tree, <<3>>, 300)\n\nroot_hash = MSSMT.root_hash(tree)\nproof = MSSMT.generate_proof(tree, <<2>>)\n\n# Verify the proof\nis_valid = MSSMT.verify_proof(root_hash, <<2>>, 200, proof)  # Returns true\n\n# Verify with incorrect value\nis_valid = MSSMT.verify_proof(root_hash, <<2>>, 250, proof)  # Returns false\n\n# Verify with incorrect key\nis_valid = MSSMT.verify_proof(root_hash, <<4>>, 200, proof)  # Returns false\n```","ref":"readme.html#generating-and-verifying-proofs"},{"type":"extras","title":"API Reference - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"","ref":"readme.html#api-reference"},{"type":"extras","title":"MSSMT.new/0 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Creates a new empty Merkle-Sum Sparse Merkle Tree.\n\n**Example:**\n\n```elixir\ntree = MSSMT.new()\n```\n\n---","ref":"readme.html#mssmt-new-0"},{"type":"extras","title":"MSSMT.insert/3 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Inserts a key-value pair into the tree.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n  - `value`: A numeric value associated with the key.\n\n**Example:**\n\n```elixir\ntree = MSSMT.insert(tree, <<1, 2, 3>>, 100)\n```\n\n---","ref":"readme.html#mssmt-insert-3"},{"type":"extras","title":"MSSMT.get/2 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Retrieves the value associated with a key.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n\n**Returns:** The value associated with the key, or `nil` if the key does not exist.\n\n**Example:**\n\n```elixir\nvalue = MSSMT.get(tree, <<1, 2, 3>>)\n```\n\n---","ref":"readme.html#mssmt-get-2"},{"type":"extras","title":"MSSMT.update/3 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Updates the value associated with a key.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n  - `value`: The new numeric value to associate with the key.\n\n**Example:**\n\n```elixir\ntree = MSSMT.update(tree, <<1, 2, 3>>, 150)\n```\n\n---","ref":"readme.html#mssmt-update-3"},{"type":"extras","title":"MSSMT.delete/2 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Deletes a key-value pair from the tree.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n\n**Example:**\n\n```elixir\ntree = MSSMT.delete(tree, <<1, 2, 3>>)\n```\n\n---","ref":"readme.html#mssmt-delete-2"},{"type":"extras","title":"MSSMT.root_hash/1 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Calculates the root hash of the tree.\n\n- **Parameters:**\n  - `tree`: The current tree.\n\n**Returns:** A 32-byte binary hash representing the root of the tree, or `nil` if the tree is empty.\n\n**Example:**\n\n```elixir\nroot_hash = MSSMT.root_hash(tree)\n```\n\n---","ref":"readme.html#mssmt-root_hash-1"},{"type":"extras","title":"MSSMT.total_sum/1 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Calculates the total sum of all values in the tree.\n\n- **Parameters:**\n  - `tree`: The current tree.\n\n**Returns:** The total sum as a number.\n\n**Example:**\n\n```elixir\ntotal_sum = MSSMT.total_sum(tree)\n```\n\n---","ref":"readme.html#mssmt-total_sum-1"},{"type":"extras","title":"MSSMT.generate_proof/2 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Generates a proof of inclusion for a given key.\n\n- **Parameters:**\n  - `tree`: The current tree.\n  - `key`: A binary representing the key.\n\n**Returns:** A list of nodes required to verify the inclusion of the key.\n\n**Example:**\n\n```elixir\nproof = MSSMT.generate_proof(tree, <<1, 2, 3>>)\n```\n\n---","ref":"readme.html#mssmt-generate_proof-2"},{"type":"extras","title":"MSSMT.verify_proof/4 - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Verifies a proof of inclusion for a given key and value.\n\n- **Parameters:**\n  - `root_hash`: The root hash of the tree.\n  - `key`: A binary representing the key.\n  - `value`: The numeric value associated with the key.\n  - `proof`: The proof generated by `generate_proof/2`.\n\n**Returns:** `true` if the proof is valid, `false` otherwise.\n\n**Example:**\n\n```elixir\nis_valid = MSSMT.verify_proof(root_hash, <<1, 2, 3>>, 150, proof)\n```","ref":"readme.html#mssmt-verify_proof-4"},{"type":"extras","title":"Development - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"To set up the project for development:\n\n1. **Clone the repository:**\n\n   ```bash\n   git clone https://github.com/AbdelStark/mssmt.git\n   cd mssmt\n   ```\n\n2. **Install dependencies:**\n\n   ```bash\n   mix deps.get\n   ```\n\n3. **Run tests:**\n\n   ```bash\n   mix test\n   ```","ref":"readme.html#development"},{"type":"extras","title":"Contributing - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"Contributions are welcome! Please feel free to submit a Pull Request.\n\nWhen contributing, please:\n\n- Write clear, concise commit messages.\n- Write tests for new functionality.\n- Ensure all tests pass before submitting.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Merkle-Sum Sparse Merkle Tree (MS-SMT) in Elixir","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2024 @AbdelStark\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}